import os
import platform
import shutil
import warnings
from datetime import datetime
from os import listdir
from os.path import dirname, join

import pkg_resources
import yaml

import sys


def load_env(path='default'):
    directory = dirname(__file__)
    if path == 'default':
        fname = join(directory, '../.env')
    else:
        fname = join(directory, 'path')
    with open(fname) as f:
        env = f.readlines()
    envars = {}
    for var in env:
        n = var.split('=')
        envars.update({n[0]: n[1].rstrip()})
    return envars


def app_config():
    try:
        with open('tir.yml') as f:
            config = yaml.load(f.read().encode('utf-8'), Loader=yaml.FullLoader)
        return config
    except FileNotFoundError:
        warnings.warn('No configuration file was found.')


def remove_list_meta(meta):
    """ Extract first and single element of list from meta dic
    :rtype: dict
    """
    if isinstance(meta, dict):
        for mk, mv in meta.items():
            if isinstance(mv, list) and len(mv) == 1:
                meta[mk] = mv[0]
    return meta


def is_windows():
    return sys.platform.startswith("win") or (sys.platform == "cli" and os.name == "nt")


def is_linux():
    return platform.system() == 'Linux'


def scan_dir(directory=None, extensions=None):
    """
    Scan a directory for files with a specific or list of extensions then return matching results

    :param directory: absolute path to directory to scan
    :type directory: str
    :param extensions: extensions to search for
    :type extensions: list
    """
    matches = []
    extensions = ['.' + ext for ext in extensions]
    files = listdir(directory)
    for file in files:
        if extensions is not None:
            for extension in extensions:
                if file.endswith(extension):
                    matches.append(join(directory, file))
        else:
            matches.append(join(directory, file))
    return matches


def mktree(path):
    if not os.path.exists(path):
        os.makedirs(path)


def url_for(route, slug=None, filename=''):
    if route == 'index':
        return '/'
    if route == 'post':
        return slug + '.html'
    elif route == 'static':
        return '{}/{}'.format('static', filename)


def format_date(value):
    date = datetime.strptime(value, "%Y-%m-%d")
    day = date.day
    if 4 <= day <= 20 or 24 <= day <= 30:
        suffix = "th"
    else:
        suffix = ["st", "nd", "rd"][day % 10 - 1]
    return date.strftime('%B %e<sup>' + suffix + '</sup> %Y')


def copytree(src, dst, symlinks=False, ignore=None):
    if not os.path.exists(dst):
        os.makedirs(dst)
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            copytree(s, d, symlinks, ignore)
        else:
            if not os.path.exists(d) or os.stat(s).st_mtime - os.stat(d).st_mtime > 1:
                shutil.copy2(s, d)


def remove_list_meta(meta):
    """ Extract first and single element of list from meta dic
    :rtype: dict
    """
    if isinstance(meta, dict):
        for mk, mv in meta.items():
            if isinstance(mv, list) and len(mv) == 1:
                meta[mk] = mv[0]
    return meta



import logging
import os
from datetime import datetime
from os import getenv, listdir
from os.path import join, normpath

import markdown
from markdown.extensions.wikilinks import WikiLinkExtension

from tir.content import ContentObject
from tir.utils import remove_list_meta

log = logging.getLogger(__name__)


class Reader(object):
    log.critical('BASE_DIR is None. Please make sure that your project base directory is set properly.')
    BASE_DIR = getenv('BASE_DIR')
    CONTENT_DIR = normpath(join(BASE_DIR, 'content'))
    POSTS_DIR = join(CONTENT_DIR, 'posts')
    RETROS_DIR = normpath(join(CONTENT_DIR, 'retrospectives'))
    LINKS_DIR = normpath(join(CONTENT_DIR, 'links/'))
    MISC_DIR = normpath(join(CONTENT_DIR, 'misc/'))

    def __call__(self):
        if not Reader.BASE_DIR:
            log.error('BASE_DIR is None. Please make sure that your project base directory is set properly.')

    @staticmethod
    def directory_scan():
        print('BASE_DIR: {}', format(Reader.BASE_DIR))
        log.error('BASE_DIR is None. Please make sure that your project base directory is set properly.')
        walk_dir = Reader.BASE_DIR

        print('walk_dir = ' + walk_dir)

        # If your current working directory may change during script execution, it's recommended to
        # immediately convert program arguments to an absolute path. Then the variable root below will
        # be an absolute path as well. Example:
        # walk_dir = os.path.abspath(walk_dir)
        print('walk_dir (absolute) = ' + os.path.abspath(walk_dir))

        for root, subdirs, files in os.walk(walk_dir):
            print('--\nroot = ' + root)
            list_file_path = os.path.join(root, 'my-directory-list.txt')
            print('list_file_path = ' + list_file_path)

            with open(list_file_path, 'wb') as list_file:
                for subdir in subdirs:
                    print('\t- subdirectory ' + subdir)

                for filename in files:
                    file_path = os.path.join(root, filename)

                    print('\t- file %s (full path: %s)' % (filename, file_path))

                    with open(file_path, 'rb') as f:
                        f_content = f.read()
                        list_file.write(('The file %s contains:\n' % filename).encode('utf-8'))
                        list_file.write(f_content)
                        list_file.write(b'\n')


class MarkdownReader(Reader):

    def __init__(self, path: str) -> None:
        self.path = path
        self.meta = None
        self.content = None
        self.yearly = False

    def read(self) -> ContentObject:
        try:
            with open('{}'.format(join(self.path)), encoding='utf8') as f:
                md = markdown.Markdown(
                    extensions=['markdown.extensions.meta', 'markdown.extensions.toc', 'markdown.extensions.footnotes',
                                'markdown.extensions.def_list', 'markdown.extensions.tables',
                                WikiLinkExtension(base_url='https://en.wikipedia.org/wiki/', end_url='')])
                html = md.convert(f.read())
                self.content = html
                if hasattr(md, 'Meta') and md.Meta:
                    meta = md.Meta
                    meta = remove_list_meta(meta)
                    if 'online' in meta and meta['online'] == 'false':
                        pass
                    if hasattr(md, 'toc'):
                        meta['contents'] = md.toc
                    self.meta = meta
                # Special handling
                # In case we process a journal entry (retrospective) we want the date to be the title
                if self.meta.get('type', None) == 'retro':
                    d = datetime.strptime(self.meta['date'], "%d-%m-%Y")
                    if self.yearly:
                        title = d.year
                    else:
                        title = '{}/{}/{}'.format(d.day, d.month, d.year)
                    self.meta['title'] = 'Retrospective {}'.format(title)
                return ContentObject(meta=self.meta, content=self.content)
        except FileNotFoundError as fnf:
            raise fnf

    @staticmethod
    def get_slugs(limit=None):
        if limit is not None:
            posts = listdir(MarkdownReader.CONTENT_DIR)[:limit]
        else:
            posts = listdir(Reader.POSTS_DIR)
        return posts



import os
import shutil
import subprocess

import pkg_resources
from jinja2 import Environment, FileSystemLoader

from tir.posts import Post
from tir.tools import is_init, _
from tir.utils import mktree, is_windows, url_for, format_date

import logging

logger = logging.getLogger(__name__)


class Tir(object):

    def __init__(self, conf):
        """Tir def

        Performs some checks on the environment before doing anything else.
        """

        self.conf = conf

        self.theme = self.conf.get('visuals').get('theme') or 'default'
        self.build_dir = self.conf['build_dir']

    def init(self):
        if is_init():
            print('A Tir project seems to be already initialized. Nothing to do.')
            return False
        print('Initializing Tir project...')
        skeleton_dirs = ['content/posts', 'content/pages']
        skeleton_files = ['package.json', 'tir.yml', 'Gruntfile.js']
        for skeleton_dir in skeleton_dirs:
            mktree(skeleton_dir)
        for skeleton_file in skeleton_files:
            shutil.copyfile(pkg_resources.resource_filename('tir', skeleton_file),
                            '{}/{}'.format(os.getcwd(), skeleton_file))
        shutil.copytree(pkg_resources.resource_filename('tir', 'assets'),
                        '{}/{}'.format(os.getcwd(), 'assets'))
        print('Installing NodeJS deps...')
        try:
            subprocess.run(['yarn', 'install'])
        except Exception:
            print('An error has occurred. Are you sure "yarn" is installed on your system?')
        print('Tir project was successfully installed.')
        return True

    def build(self):
        if not is_init():
            print('Project does not seem to be initialized. Type "tir init" to create a Tir project here.')
            return False
        try:
            pkg_visuals_dir = pkg_resources.resource_filename('tir', 'visuals')
            tpl_visuals_dir = os.path.join(pkg_visuals_dir, self.theme)
            logger.debug('Selected template: %s', tpl_visuals_dir)
            print('Replace assets files')
            assets_src_dir = os.path.join(tpl_visuals_dir, 'assets')
            assets_target_dir = '{}/{}'.format(os.getcwd(), 'assets')
            print('Removing {}'.format(assets_target_dir))
            if os.path.exists(assets_target_dir):
                shutil.rmtree(assets_target_dir)
            shutil.copytree(assets_src_dir, assets_target_dir)
            print('Assets successfully updated')
            env = Environment(loader=FileSystemLoader(os.path.join(tpl_visuals_dir, 'templates')))
            post_tpl = env.get_template('post.html')
            index_tpl = env.get_template('home.html')
            env.globals['url_for'] = url_for
            env.globals['_'] = _
            env.globals['format_date'] = format_date
            env.globals['config'] = self.conf
            print('Building...')
            slugs = Post.get_slugs()
            for slug in slugs:
                slug = slug.replace('.md', '')
                p = Post()
                x = p.read(slug)
                target_path = '%s/%s.html' % (self.build_dir, slug)
                mktree(self.build_dir)
                with open(target_path, 'w', encoding='utf-8') as fh:
                    head = {'title': x.meta['title'],
                            'description': x.meta['subtitle']}
                    fh.write(post_tpl.render(
                        post=p,
                        head=head,
                    ))
                print('Compiling {}...'.format(target_path))

            # Building index page
            print('Building index page...')
            with open(self.build_dir + '/index.html', 'w', encoding='utf-8') as fh:
                p = Post()
                x = p.read('index', dir_path=Post.MISC_DIR)
                head = {'title': 'ouafi.net',
                        'description': 'Dans un monde fou, toute forme d\'écriture est un remède psychiatrique'}
                fh.write(index_tpl.render(
                    content={'intro': x},
                    post=p,
                    head=head
                ))

            print('Building static files...')
            if not is_windows():
                subprocess.run(['yarn', 'build'])
            else:
                subprocess.Popen(['yarn', 'build'], shell=True)
            print('Build was successful.')
            return True

        except Exception as e:
            raise e



import os
from datetime import datetime
from os import getenv, listdir
from os.path import join, normpath

import markdown
from markdown.extensions.wikilinks import WikiLinkExtension

from tir.utils import remove_list_meta


class Post(object):
    d = os.getcwd()
    BASE_DIR = getenv('BASE_DIR', d)
    CONTENT_DIR = normpath(join(BASE_DIR, 'content'))
    POSTS_DIR = join(CONTENT_DIR, 'posts')
    RETROS_DIR = normpath(join(CONTENT_DIR, 'retrospectives'))
    LINKS_DIR = normpath(join(CONTENT_DIR, 'links/'))
    MISC_DIR = normpath(join(CONTENT_DIR, 'misc/'))

    def __init__(self):
        self.meta = None
        self.content = None
        self.yearly = False

    def read(self, el, dir_path=POSTS_DIR, ext='.md'):
        try:
            with open('{}'.format(join(dir_path, el + ext)), encoding='utf8') as f:
                md = markdown.Markdown(
                    extensions=['markdown.extensions.meta', 'markdown.extensions.toc', 'markdown.extensions.footnotes',
                                'markdown.extensions.def_list', 'markdown.extensions.tables',
                                WikiLinkExtension(base_url='https://en.wikipedia.org/wiki/', end_url='')])
                html = md.convert(f.read())
                self.content = html
                if hasattr(md, 'Meta') and md.Meta:
                    meta = md.Meta
                    meta = remove_list_meta(meta)
                    if 'online' in meta and meta['online'] == 'false':
                        pass
                    if hasattr(md, 'toc'):
                        meta['contents'] = md.toc
                    self.meta = meta
                # Special handling
                # In case we process a journal entry (retrospective) we want the date to be the title
                if dir_path == Post.RETROS_DIR:
                    d = datetime.strptime(self.meta['date'], "%d-%m-%Y")
                    if self.yearly:
                        title = d.year
                    else:
                        title = '{}/{}/{}'.format(d.day, d.month, d.year)
                    self.meta['title'] = 'Retrospective {}'.format(title)
                return self
        except FileNotFoundError as fnf:
            raise fnf

    @staticmethod
    def get_slugs(limit=None):
        if limit is not None:
            posts = listdir(Post.POSTS_DIR)[:limit]
        else:
            posts = listdir(Post.POSTS_DIR)
        return posts

    @staticmethod
    def get_intro():
        post = Post()
        return post.read(Post.MISC_DIR + '/intro.md')

    @staticmethod
    def read_all(slugs):
        posts = []
        for slug in slugs:
            post = Post()
            p = post.read(Post.POSTS_DIR, slug.replace('.md', ''))
            posts.append(p)
        return posts



"""
todos:
* create html directory if doesn't exist
* format date in meta
* check if toc is rendered
* get stats at the end of build
* make configurable options: github, website, url, http/https, templates
* remove personal files and replace with samples (logo, ...)
* don't remove tir.yml if it already exists
"""

import logging
import os
import shutil
from datetime import datetime

log = logging.getLogger(__name__)


def url_for(route, slug=None, filename=''):
    if route == 'index':
        return '/'
    if route == 'post':
        return slug + '.html'
    elif route == 'static':
        return '{}/{}'.format('static', filename)


def format_date(date, date_format="%d/%m/%Y", suffix=False):
    date = datetime.strptime(date, '%Y-%m-%d')
    return date.strftime(date_format)


def copytree(src, dst, symlinks=False, ignore=None):
    if not os.path.exists(dst):
        os.makedirs(dst)
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            copytree(s, d, symlinks, ignore)
        else:
            if not os.path.exists(d) or os.stat(s).st_mtime - os.stat(d).st_mtime > 1:
                shutil.copy2(s, d)


def mktree(path):
    if not os.path.exists(path):
        os.makedirs(path)


def _(text):
    return text


def is_init():
    return os.path.exists('tir.yml')



import argparse
import logging

from tir import Tir
from tir.settings import load_settings

log = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser(description='Tir CLI', formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('command', nargs='?', help='command to execute',
                        choices=['init', 'build'])

    args = parser.parse_args()

    config = load_settings()
    tir = Tir(config)

    if args.command == 'init':
        tir.init()
    if args.command == 'build':
        tir.build()



from os import walk
from os.path import relpath, join

from setuptools import find_packages, setup

version = "0.3.0"
setup(
    name='tir',
    version=version,
    author='kvfi',
    author_email='mail@mail.net',
    url='https://ouafi.net/Tir.html',
    license='https://www.gnu.org/licenses/gpl-3.0.en.html',
    description=(
        'Minimalist static site generator'
    ),
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    setup_requires=['wheel'],
    install_requires=[
        'jinja2',
        'markdown',
        'pyyaml'
    ],
    package_data={
        'tir': [relpath(join(root, name), 'tir')
                for root, _, names in walk(join('tir', 'data')) and walk(join('tir', 'visuals'))
                for name in names]
    },
    entry_points={
        'console_scripts': [
            'tir = tir.cli:main'
        ]
    },
    packages=find_packages()
)



import logging
import warnings

from yaml import Dumper, FullLoader, load, dump

logger = logging.getLogger(__name__)

_DEFAULT_SETTINGS = {
    'build_dir': 'html',
    'visuals': {
        'theme': 'default'
    },
}


def load_settings(override: dict = None) -> dict:
    try:
        with open('tir.yml', 'r', encoding='UTF-8') as f:
            content = f.read()
            cfg = load(content, Loader=FullLoader) or {}
    except FileNotFoundError:
        warnings.warn('No configuration file was found.')

    merged_cfg = {**cfg, **_DEFAULT_SETTINGS}

    print(merged_cfg)

    if override is not None:
        merged_cfg = {**merged_cfg, **override}

    with open('tir.yml', 'w+', encoding='UTF-8') as f:
        f.write(dump(merged_cfg, Dumper=Dumper))

    return merged_cfg
